# -*- coding: utf-8 -*-
###########################################################################
# 
# Eole NG
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################

"""generation de code cheetah sur des valeurs creole multivaluees"""

import re, sys
from copy import copy
from creole.cfgparser import EoleDict
from creole import config
from era.backend.template import Template

def eole_dict():
    """recuperation du dictionnaire creole
    afin de recuperer l'info sur les groupe et
    les variables maitre-esclaves
    """
    if sys.platform == 'win32': dico = EoleDict(templatedir=config.templatedir,logfile='nul')
    else: dico = EoleDict(templatedir=config.templatedir,logfile='/dev/null')
    dico.read_dir(config.eoledirs)
    return dico

creole_dico = None

def get_groups():
    """ dictionnaire des groupes creole
    {maitre:[esclave1, esclave2]]}
    """
    return creole_dico.groups

def is_multi(varname):
    """permet de savoir si une variable creole est multiple ou non
    """
    for var, value in creole_dico.variables.items():
        if varname == var:
            return value.multi
    raise Exception, "la variable %s n'est pas trouvee" % varname

class Multivar:
    """generation de code cheetah sur des valeurs multivaluees"""
    def __init__(self, lines=[], creole_dico=None):
        """:lines: lignes iptables generees par le compilateur"""
        self.lines = lines
        if creole_dico == None:
            globals()['creole_dico'] = eole_dict()
        else:
            globals()['creole_dico'] = creole_dico
            
    def _getcreolevar_inline(self, line):
        """:return: la liste des variables creole dans une ligne
        """
        # regexp de motif %%mavariable[] ou mavariable[1]
        monovarregexp = re.compile('%%(\w*)\[[0-9]*\]')
        # regexp de motif %%mavariable
        multivarregexp = re.compile('%%(\w*)')

        # toute les variables templates sont parsees
        multivars = re.findall(multivarregexp, line) 
        for var in multivars:
            # cette variable ne doit pas etre suivie de crochets
            if var not in re.findall(monovarregexp, line):
                yield var

    def _ismulti(self, line):
        """permet de savoir si une ligne est multi-valuee ou non """
        for var in self._getcreolevar_inline(line):
            if is_multi(var): return True
        else: return False

    def _is_mastervar(self, line):
        """savoir si une variable de la ligne est maitre  """
        for var in self._getcreolevar_inline(line):
            if var in get_groups():
                return True
        else: return False

    def _validate_group(self, master, vars):
        """
        verifie que les variables presentes sont toutes dans le mme groupe
        celui de la variable (eventuelle) maitre
        :master: variable maitre
        :var: variables de la ligne 
        """
        for var in vars:
            if not get_groups().has_key(master):
                raise Exception, "la variable %s a instancier n'est pas dans le dictionnaire creole"% (master)
                
            if var not in get_groups()[master]:
                raise Exception, "la variable creole multiple %s n'est pas dans le groupe de la variable maitre %s" % \
                      (var, master)
            
    def process(self):
        """collecte les valeurs multivaluees dans le dictionaire creole
        """
        for line in self.lines:
            if self._ismulti(line):
                # retour au templating de la lib standard
                tpline = copy(line)
                creolevars = list( self._getcreolevar_inline(line) )
                master = None
                # s'il y a une variable maitre dans la ligne
                if self._is_mastervar(line):
                    for var in self._getcreolevar_inline(line):
                        # reperons une variable maitre
                        if var in get_groups():
                            creolevars.remove(var)
                            master = var
                            slaves = get_groups()[master]

                    # enlevons les variables non multi valuees
                    non_multi = []
                    tmp_creolevars = copy(creolevars)
                    for var in tmp_creolevars:
                        if not is_multi(var):
                            non_multi.append(var)
                            creolevars.remove(var)
                    # validons la coherence de la ligne (un seul groupe)
                    self._validate_group(master, creolevars)
                    # on ne templatise que les variables multivaluees et la variable maitre
                    templatevars = copy(creolevars)
                    templatevars.append(master)
                    # templating de la lib standard
                    for var in templatevars:
                        tpline = tpline.replace("%%" + var, "%("+ var + ")s")
                    for var in slaves:
                        tpline = tpline.replace("%%" + var, "%("+ var + ")s")
                    # doubles % pour les variables non multi-valuees
                    for var in non_multi:
                        tpline = tpline.replace("%%" + var, "%%%%" + var)

                    # generation du template cheetah
                    tpl = Template(master=master, slaves=slaves, template=tpline)
                    # pour qu'il n'y ait qu'une seule ligne a chaque fois -> split
                    newline = str(tpl).split('\n')
                    for n in newline:
                        yield n + '\n'
            else:
                # pas de variables multivaluees : pas de traitement 
                yield line


