# -*- coding: UTF-8 -*-
###########################################################################
# 
# Eole NG  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr 
#  
###########################################################################
"""
    Controlleur (mvc) au niveau de l'application
"""
import gtk
from era.noyau.path import setup_path
setup_path()
from gtkmvc import Controller

class OptionCtrl(Controller):
    def __init__(self, model, view):
        Controller.__init__(self, model, view)

    def register_view(self, view):
        """
            formatage du treeview
        """
        treeview = view.opt_treeview
        treeview.set_model(self.model.list_store) 
        renderer = gtk.CellRendererText()
        tog_renderer = gtk.CellRendererToggle()
        # double-click
        tog_renderer.connect("toggled", self.event_toggle, self.model.list_store)
        col_name = gtk.TreeViewColumn(_('option label'), renderer, text=0)
        col_name.set_expand(True)
        col_tog = gtk.TreeViewColumn(_('option active'), tog_renderer, active=1)
        treeview.append_column(col_name)
        treeview.append_column(col_tog)
        
    # ____________________________________________________________
    # values
    def property_options_value_change(self, model, old, new):
        """
            si les options du xml ont changées, recharger le modele
        """
        self.model.reload_liststore()
        
    def property_options_after_change(self, model, instance, name, res,
                                      args, kwargs):
        """
            appel a la méthode ``update()`` de library_store.options
        """
        self.model.reload_liststore()
    # ____________________________________________________________
    # signals
    def event_toggle(self, cell, path, model):
        """
            double-click on the toggle renderer callback
        """
        model[path][1] = not model[path][1]

    def on_options_dialog_delete_event(self, wdgt, evnt):
        self.view.get_top_widget().hide_all()
        #self.model.reload_liststore()
        return True

#    def on_valid_options_dialog_clicked(self, wdgt):
#        self.view.get_top_widget().hide_all()
#        self.save_options()
#        return True
#        
#    def on_close_options_dialog_clicked(self, wdgt):
#        self.view.get_top_widget().hide_all()
#        return True

    def on_options_dialog_response(self, wdgt, evnt):
        """
            callback on close pour l'action *hide*
        """
        if evnt == gtk.RESPONSE_OK:
            # parser le liststore et enregistrer dans le modele
            self.update_options()
        else:
            # recharger  le treeview avec les options par defaut
            self.model.reload_liststore()
 
        self.view.get_top_widget().hide_all()
        return True

    def on_options_dialog_close(self, wdgt, evnt):
        self.view.get_top_widget().hide_all()
        return True

    def update_options(self):
        """
            contenu du treeview d'edition -> ``library_store.options``
        """
        self.view.opt_treeview.get_model().foreach(self.update_options_cb,[])
        return 
        
    def update_options_cb(self, model, path, current_iter, data):
        """
            callback du foreach du modele du treeview
        """
        name = str(model.get_value(current_iter,0))
        flag = model.get_value(current_iter,1)
        if flag == True:
            txt = '1'
        else:
            txt = '0'
        # appel à la méthode ``update`` pour passer par ``after_change``
        buf_dict = {name:txt}
        self.model.options.update(buf_dict)
        
